@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; c-file-style: "bsd"; c-basic-offset: 4; fill-column: 108; indent-tabs-mode: nil; -*-
  *
- * Copyright (c) 2004-2024 Apple Inc. All rights reserved.
+ * Copyright (c) 2004-2025 Apple Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,7 +26,7 @@
 #include <sys/socket.h>         // Needed for socket() etc.
 #include <netinet/in.h>         // Needed for sockaddr_in
 #include <syslog.h>
-#include <sys/fcntl.h>
+#include <fcntl.h>
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
 #include <time.h>
@@ -281,7 +281,6 @@ mDNSexport void mDNSPlatformWriteDebugMsg(const char *msg)
 }
 #endif
 
-#if !MDNSRESPONDER_SUPPORTS(APPLE, OS_LOG)
 mDNSexport void mDNSPlatformWriteLogMsg(const char *ident, const char *buffer, mDNSLogLevel_t loglevel)
 {
 
@@ -317,7 +316,6 @@ mDNSexport void mDNSPlatformWriteLogMsg(const char *ident, const char *buffer, m
         }
     }
 }
-#endif // !MDNSRESPONDER_SUPPORTS(APPLE, OS_LOG)
 
 mDNSexport mDNSBool mDNSPosixTCPSocketSetup(int *fd, mDNSAddr_Type addrType, mDNSIPPort *port, mDNSIPPort *outTcpPort)
 {
@@ -327,7 +325,7 @@ mDNSexport mDNSBool mDNSPosixTCPSocketSetup(int *fd, mDNSAddr_Type addrType, mDN
     mDNSu32 lowWater = 15384;
 
     sock = socket(sa_family, SOCK_STREAM, IPPROTO_TCP);
-    if (sock < 3)
+    if (sock < 0)
     {
         if (errno != EAFNOSUPPORT)
         {
@@ -778,7 +776,7 @@ mDNSlocal void getLocalTimestampFromTimeval(const struct timeval *const tv,
     strftime(timeZoneStr, sizeof(timeZoneStr), "%z", &localTime);
 
     // Construct the final timestamp with the milliseconds.
-    snprintf(outBuffer, bufferLen, "%s.%03u%s", dateTimeStr, (mDNSs32)tv->tv_usec / (mDNSs32)USEC_PER_MSEC,
+    snprintf(outBuffer, bufferLen, "%s.%03d%s", dateTimeStr, (mDNSs32)tv->tv_usec / (mDNSs32)USEC_PER_MSEC,
              timeZoneStr);
 }
 
