@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2022 Apple Inc. All rights reserved.
+ * Copyright (c) 2003-2025 Apple Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -28,10 +28,8 @@
 #include <netinet/in.h>
 #include "DNSCommon.h"          // For mDNSPlatformInterfaceIndexfromInterfaceID().
 
-#if MDNSRESPONDER_SUPPORTS(APPLE, DNSSECv2)
-#include "dnssec.h"
-#include "dnssec_obj_context.h"
-#endif
+
+#include "mdns_strict.h"
 
 extern mDNS mDNSStorage;        // We need to pass the address of this storage to the lower-layer functions
 
@@ -647,12 +645,6 @@ mDNSlocal void DNSServiceQueryRecordResponse(mDNS *const m, DNSQuestion *questio
     ConvertDomainNameToCString(answer->name, fullname);
 
     DNSServiceFlags flags = AddRecord ? kDNSServiceFlagsAdd : (DNSServiceFlags)0;
-#if MDNSRESPONDER_SUPPORTS(APPLE, DNSSECv2)
-    if (dns_question_is_dnssec_requestor(question))
-    {
-        flags |= dns_service_flags_init_with_dnssec_result(question, answer);
-    }
-#endif
 
     x->callback((DNSServiceRef)x, flags, 0, kDNSServiceErr_NoError, fullname, answer->rrtype, answer->rrclass,
                 answer->rdlength, answer->rdata->u.data, answer->rroriginalttl, x->context);
@@ -694,9 +686,6 @@ DNSServiceErrorType DNSServiceQueryRecord
     x->q.ForceMCast           = (flags & kDNSServiceFlagsForceMulticast) != 0;
     x->q.ReturnIntermed       = (flags & kDNSServiceFlagsReturnIntermediates) != 0;
     x->q.SuppressUnusable     = (flags & kDNSServiceFlagsSuppressUnusable) != 0;
-#if MDNSRESPONDER_SUPPORTS(APPLE, DNSSECv2)
-    x->q.enableDNSSEC         = dns_service_flags_enables_dnssec(flags);
-#endif
     x->q.AppendSearchDomains  = 0;
     x->q.TimeoutQuestion      = 0;
     x->q.WakeOnResolve        = 0;
@@ -722,12 +711,22 @@ fail:
 // DNSServiceGetAddrInfo
 //
 
-static void DNSServiceGetAddrInfoDispose(mDNS_DirectOP *op)
+static void DNSServiceGetAddrInfoForget(mDNS_DirectOP_GetAddrInfo **const op)
 {
-    mDNS_DirectOP_GetAddrInfo *x = (mDNS_DirectOP_GetAddrInfo*)op;
-    if (x->a.ThisQInterval >= 0) mDNS_StopQuery(&mDNSStorage, &x->a);
-    if (x->aaaa.ThisQInterval >= 0) mDNS_StopQuery(&mDNSStorage, &x->aaaa);
-    mDNSPlatformMemFree(x);
+    if (*op)
+    {
+        mDNS_DirectOP_GetAddrInfo *const x = *op;
+        if (x->a.ThisQInterval >= 0) mDNS_StopQuery(&mDNSStorage, &x->a);
+        if (x->aaaa.ThisQInterval >= 0) mDNS_StopQuery(&mDNSStorage, &x->aaaa);
+        mDNSPlatformMemFree(x);
+        *op = mDNSNULL;
+    }
+}
+
+static void DNSServiceGetAddrInfoDispose(mDNS_DirectOP *const op)
+{
+    mDNS_DirectOP_GetAddrInfo *tmp = (mDNS_DirectOP_GetAddrInfo *)op;
+    DNSServiceGetAddrInfoForget(&tmp);
 }
 
 mDNSlocal void DNSServiceGetAddrInfoResponse(mDNS *const m, DNSQuestion *question,
@@ -876,9 +875,6 @@ DNSServiceErrorType DNSSD_API DNSServiceGetAddrInfo(
     x->a.ForceMCast           = (inFlags & kDNSServiceFlagsForceMulticast) != 0;
     x->a.ReturnIntermed       = (inFlags & kDNSServiceFlagsReturnIntermediates) != 0;
     x->a.SuppressUnusable     = (inFlags & kDNSServiceFlagsSuppressUnusable) != 0;
-#if MDNSRESPONDER_SUPPORTS(APPLE, DNSSECv2)
-    x->a.enableDNSSEC         = dns_service_flags_enables_dnssec(inFlags);
-#endif
     x->a.AppendSearchDomains  = 0;
     x->a.TimeoutQuestion      = 0;
     x->a.WakeOnResolve        = 0;
@@ -907,6 +903,7 @@ DNSServiceErrorType DNSSD_API DNSServiceGetAddrInfo(
 
 fail:
     LogMsg("DNSServiceGetAddrInfo(\"%s\", %d) failed: %s (%ld)", inHostName, inProtocol, errormsg, err);
+    DNSServiceGetAddrInfoForget(&x);
     return(err);
 }
 
